import requests
import json


def okapi_send_request(okapi_login, request_body, url_endpoint, max_retries=3):
    # okapi_send_request() Send request to okapi platform
    #
    #   Inputs
    #       okapi_login - Dict, containing at least URL, options and Token for
    #       okapi. Can be obtained using OkapiInit().
    #       request_body - The body of the request (thus: the message).
    #       url_endpoint - the url, where-to send the request
    #
    #   Outputs
    #       request - dict containing the request_id needed to identify the
    #                 result on OKAPI servers.
    #       error - Dict containing error information. Always check
    #               error['status'] If it is 'FATAL' something went very wrong,
    #               'WARNING's are less critical and 'NONE' or 'INFO' are no
    #               concern. error['message'] gives some explanation on the
    #               status, error['web_status'] gives the http response.

    # check the type that is requested

    # init
    request = dict()
    response = dict()
    error = dict()
    error['message'] = 'NONE'
    error['status'] = 'NONE'
    error['web_status'] = 0

    url = okapi_login["url"] + url_endpoint
    retries = 1
    while(retries <= max_retries):
        try:
            response = requests.post(url, data=json.dumps(request_body),
                                    headers=okapi_login['header'], timeout=5)
            # raise for status
            response.raise_for_status()
            break

        except requests.exceptions.HTTPError as e:
            print("Exception: " + str(e))
            print("Response Body: {}".format(response.json()))

            # if we got a 500, we received an internal error.This we would like to
            # look at
            if (response.status_code == 500):
                response_json = response.json()
                status = response_json['state_msg']
                error['message'] = status['text']
                error['status'] = status['type']
            else:
                error['message'] = 'Got HTTPError when sending request: ' + str(e)
                # DEBUG
                # print("HTTP Response " + str(response.json()))
                error['status'] = 'FATAL'
            error['web_status'] = response.status_code
            return request, error
        except requests.exceptions.Timeout as e:
            if(retries == max_retries):     
                error['message'] = 'Got timeout when sending request: ' + str(e)
                error['status'] = 'FATAL'
                error['web_status'] = 408
                return request, error
            retries += 1
            continue 
        except requests.exceptions.RequestException as e:
            error['message'] = 'Got unknown exception (Wrong url?): ' + str(e)
            error['status'] = 'FATAL'
            error['web_status'] = 520  # non-standard
            return request, error

    # apparently, all when smoothly. Get the responses
    # DEBUG
    # print("HTTP Response " + str(response.json()))
    response_json = response.json()

    # fill error
    status = response_json['status']   # state_msg = response_json['state_msg']
    error['message'] = status['text']
    error['status'] = status['type']
    error['web_status'] = response.status_code

    # fill request -- depending on what has been called!
    request['id'] = response_json['request_id']

    return request, error



import requests
import time


def okapi_init(url, username, password, max_retries=3):
    # okapi_init() Performs the login and initialization. By default, all
    # possible scopes are requested.
    #
    # Inputs:
    #     url - The url, where OKAPI is running.
    #     username - the username to access OKAPI_Orbits' services
    #     password - the password to access OKAPI_Orbits' services
    #
    # Outputs:
    #     okapi_login - Dict, containing URL, Token, accessTime and options.
    #                   Send to all other routines. NOTE: Is valid for 1 day
    #     error - Dict containing error information. Always check
    #             error['status'] If it is 'FATAL' something went very wrong,
    #             'WARNING's are less critical and 'NONE' or 'INFO' are no
    #             concern. error['message'] gives some explanation on the
    #             status, error['web_states'] gives the http response.
    #

    # init
    error = dict()
    okapi_login = dict()
    error['message'] = 'NONE'
    error['status'] = 'NONE'
    error['web_status'] = 0

    request_token_payload = {
        "grant_type": "password",
        "username": username,
        "password": password,
        "audience": "https://api.okapiorbits.space/picard",
        "scope": ('neptune_propagation neptune_propagation_request '
                  'pass_predictions pass_prediction_requests '
                  'pass_predictions_long pass_prediction_requests_long'),
        "client_id": "jrk0ZTrTuApxUstXcXdu9r71IX5IeKD3",
    }

    retries = 1
    while(retries <= max_retries):
        try:
            url_auth = "https://okapi-development.eu.auth0.com/oauth/token"
            okapi_login_token_response = requests.post(url_auth,
                                                    data=request_token_payload,
                                                    timeout=5)
            okapi_login_token_response.raise_for_status()
            okapi_login_token = okapi_login_token_response.json()

            okapi_login = {
                "token": okapi_login_token,
                "url": url,
                "accessTime": str(time.time()),
                "header": {
                    "Content-Type": "application/json",
                    "Accept": "application/json",
                    #"access_token": okapi_login_token["access_token"], # Legacy
                    "Authorization": "Bearer " + okapi_login_token["access_token"],
                    "expires_in": str(okapi_login_token["expires_in"]),
                    "token_type": okapi_login_token["token_type"],
                    "scope": okapi_login_token["scope"]
                }
            }
            break
        except requests.exceptions.HTTPError as err:
            print(err)
            # we now that 403 is probably wrong password:
            if (okapi_login_token_response.status_code == 403):
                response = okapi_login_token_response.json()
                error['message'] = ('Got error from Auth0: ' + response['error'] +
                                    ': ' + response['error_description'])
                error['status'] = 'FATAL'
                error['web_status'] = okapi_login_token_response.status_code
            else:
                error['message'] = 'Got HTTP Error when sending request. '
                error['status'] = 'FATAL'
                error['web_status'] = okapi_login_token_response.status_code
                print("Error occured: {}".format(err))
            return okapi_login, error
        except requests.exceptions.Timeout as err:
            if(retries == max_retries):
                print(err)
                error['message'] = 'Got timeout when sending request. '
                error['status'] = 'FATAL'
                error['web_status'] = 408  # correct timeout?
                return okapi_login, error
            retries += 1
            continue
        except requests.exceptions.RequestException as err:
            print(err)
            error['message'] = 'Got unknown exception. '
            error['status'] = 'FATAL'
            error['web_status'] = 520  # non-standard
            return okapi_login, error

    return okapi_login, error



import requests

def okapi_get_objects(okapi_login, url_endpoint, sub_id = '', max_retries=3):
    # okapi_get_objects() Get objects from an endpoint
    #
    #   Inputs
    #       okapi_login - Dict, containing at least URL, options and Token for
    #       OKAPI. Can be obtained using OkapiInit().
    #       url_endpoint - adress, from which the results shall be retrieved
    #
    #   Outputs
    #       results - dict, containing the results from the request
    #       error - Dict containing error information. Always check
    #               error['status'] If it is 'FATAL' something went very wrong,
    #               'WARNING's are less critical and 'NONE' or 'INFO' are no
    #               concern. error['message'] gives some explanation on the
    #               status, error['web_states'] gives the http response.
    #   Examples
    #      Get all conjunctions for one user
    #       conjunctions, error = okapi_get_objects(okapi_login,'conjunctions')
    #      Get all CDMs for the first conjunction of conjunctions
    #       cdms, error = okapi_get_objects(okapi_login,'conjunctions/{}/cdms',conjunctions["elements"][0]["conjunction_id"])
    #

    # init
    result = dict()
    result_dict = dict()
    error = dict()
    error['message'] = 'NONE'
    error['status'] = 'NONE'
    error['web_status'] = 0

    # check what kind of object is requested
    if (sub_id == ''):
        url = okapi_login["url"] + url_endpoint
    else:
        url = okapi_login["url"] + url_endpoint.format(sub_id)

    retries = 1
    while(retries <= max_retries):
        try:
            result["service"] = url_endpoint
            response = requests.get(url, headers=okapi_login["header"], timeout=5)

            # raise for status!
            response.raise_for_status()

            # extract result and stuff: Loop over all objects in the list that was
            # nicely provided to us
            result = response.json()

            # Get the state msgs and stuff
            current_result_dict = result
            if ('status' in current_result_dict):
                state_msg = current_result_dict['status']
                error['status'] = state_msg['type']
                error['message'] = state_msg['text']

            error['web_status'] = response.status_code

            # Finally: Check, if status is 202 which means "results might not be
            # complete"
            if ((response.status_code == 202) and (error['status'] != 'FATAL')):
                error['status'] = 'WARNING'
                error['message'] = 'Result has been accepted but not been fully processed yet.'

            break

        except requests.exceptions.HTTPError as e:
            print("Exception: " + str(e))
            print("Response Body: {}".format(response.json()))
            # need to get the response from the result to send the error
            try:
                result = response.json()
                result_dict = result[0]
            except:
                result = dict()
                result_dict = dict()

            if ('state_msg' in result_dict):
                state_msg = result_dict['state_msg']
                error['status'] = state_msg['type']
                error['message'] = state_msg['text']
            elif('state_msgs' in result_dict):
                result_dict = result[1]
                state_msgs = result_dict['state_msgs']

                state_msgs_temp = state_msgs[0]
                error['status'] = state_msgs_temp['type']
                error['message'] = state_msgs_temp['text']

            error['web_status'] = response.status_code
            return result, error
        except requests.exceptions.Timeout:
            if(retries == max_retries):
                error['message'] = 'Got timeout when sending request. '
                error['status'] = 'FATAL'
                error['web_status'] = 408
                return result, error
            retries += 1
            continue
        except requests.exceptions.RequestException:
            error['message'] = 'Got unknown exception. '
            error['status'] = 'FATAL'
            error['web_status'] = 520  # non-standard
            return result, error

    return result, error



import requests
import json
# from okapi_init import okapi_init
# from okapi_send_request import okapi_send_request
# from okapi_get_result import okapi_get_result
import time

# This is a wrapper class that is supposed to make it easy to try the python connector
def okapi_send_request_and_wait_for_result(okapi_login, request_body, url_endpoint_requests, url_endpoint_results, max_poll_time):
    # okapi_send_request_and_wait_for_result() Send a request to the OKAPI Platform and wait for the results
    #
    #   Inputs
    #       okapi_login - Dict, containing at least URL, options and Token for
    #       okapi. Can be obtained using OkapiInit().
    #       request_body - The body of the request (thus: the message).
    #       url_endpoint_requests - the url, where-to send the request
    #       url_endpoint_results - the url, where-from get the result
    #       max_poll_time - the maximum time (in seconds) to wait for the result. Measurement is NOT exact
    #
    #   Outputs
    #       result - dict, containing the result from the request
    #       error - Dict containing error information. Always check
    #               error['status'] If it is 'FATAL' something went very wrong,
    #               'WARNING's are less critical and 'NONE' or 'INFO' are no
    #               concern. error['message'] gives some explanation on the
    #               status, error['web_states'] gives the http response.

    # send the request to the server
    request, error = okapi_send_request(okapi_login, request_body, url_endpoint_requests)


    if (error['status'] == 'FATAL'):
        print(error)
        exit()
    elif(error['status'] == 'WARNING'):
        print(error)

    # Wait the specified time for the result. Max time makes sure that we cannot get stuck
    counter = 0
    error['web_status'] = 202
    while (counter < int(max_poll_time)) and (error['web_status'] == 202):

    # get the results from the request
        result, error = okapi_get_result(
            okapi_login, request,
            url_endpoint_results)
        if (error['status'] == 'FATAL'):
            print(error)
            exit()
        elif(error['status'] == 'WARNING'):
            print(error)

        # we wait a second, to not trigger some DOD on the server ;-)
        time.sleep(1)

        counter += 1

    return result, error



from .okapi_add_object import okapi_add_object
from .okapi_change_object import okapi_change_object
from .okapi_delete_object import okapi_delete_object
from .okapi_get_objects import okapi_get_objects
from .okapi_init import okapi_init
from .okapi_send_request_and_wait_for_result import okapi_send_request_and_wait_for_result
from .okapi_send_request import okapi_send_request
from .okapi_wait_and_get_result import okapi_wait_and_get_result
from .okapi_get_result import okapi_get_result





import requests
import time


class Okapi:



  def get_status_severity(status_type):
    if status_type == "NONE":
      return 0
    elif status_type == "NORMAL":
      return 100
    elif status_type == "REMARK":
      return 200
    elif status_type == "WARNING":
      return 300
    elif status_type == "FATAL":
      return 400
    else:
      print("WARNING: Unkown status type \"" + status_type + "\"")
      return 1000



  #def merge_statuses(source_status, target_status):
  #  result_status = target_status
  #  if get_status_severity(source_status["type"]) > get_statu


  def handle_rest_call(func, is_login_call = False, max_retries=3):

    connector_status = dict()
    connector_status["text"] = "All ok"
    connector_status["type"] = "NORMAL"
    connector_status["web_status"] = 0
    
    retries = 1
    while(retries <= max_retries):
      try:
        response = func()
        response.raise_for_status()
        break

      except requests.exceptions.HTTPError as err:
        #print(err)
        # we know that 403 is probably wrong password:
        if (is_login_call and response.status_code == 403):
          try:
            connector_response = response.json()
            connector_status['text'] = "Got error from Auth0: " \
              + response['error'] + ': ' + response['error_description']
            connector_status['type'] = 'FATAL'
            connector_status['web_status'] = response.status_code
            break
          except:
            #print("Could not convert response data to json")
            connector_status['text'] = "Got unknown error from Auth0: " + str(err)
            connector_status['type'] = 'FATAL'
            connector_status['web_status'] = response.status_code
            break
        else:
          connector_status['text'] = 'Got HTTPError when sending request: ' \
            + str(err)
          connector_status['type'] = 'FATAL'
          connector_status['web_status'] = response.status_code
          break
      except requests.exceptions.Timeout as err:
        if(retries == max_retries):
          connector_status['text'] = 'Got timeout when sending request: ' + str(err)
          connector_status['type'] = 'FATAL'
          connector_status['web_status'] = 408
        retries += 1
        continue
      except requests.exceptions.RequestException as err:
        connector_status['text'] = 'Got unknown exception (Wrong url?).'
        connector_status['type'] = 'FATAL'
        connector_status['web_status'] = 520  # non-standard
        connector_status['error'] = err
        break
      break
    
    connector_response = dict()
    try:
      connector_response["actual_response"] = response.json()
    except:
      #print("Could not convert response data to json")
      connector_response["actual_response"] = response.content
    connector_response["status"] = connector_status
    #print("auch hier: " + str(connector_response))
    return connector_response


  def __init__(self):

    self.initialized = False


  def init(self, username, password, url 
    = "http://platform.okapiorbits.com"):
    
    # init
    self.okapi_login = dict()
    self.url = url
    #status = dict()
    #status["text"] = "NONE"
    #status["type"] = "NONE"
    #status["web_status"] = 0

    #if self.initialized:
    #    status["type"] = "FATAL"
    #    status["message"] = "

    request_token_payload = \
    {
      "grant_type": "password",
      "username": username,
      "password": password,
      "audience": "https://api.okapiorbits.space/picard",
      "scope": ("neptune_propagation neptune_propagation_request "
        "pass_predictions pass_prediction_requests "
        "pass_predictions_long pass_prediction_requests_long"),
      "client_id": "jrk0ZTrTuApxUstXcXdu9r71IX5IeKD3",
    }

    url_auth = "https://okapi-development.eu.auth0.com/oauth/token"
    connector_response = Okapi.handle_rest_call(lambda: requests.post(url_auth,
      data = request_token_payload, timeout = 5), True)


    okapi_login_token = connector_response["actual_response"]

    #try:
    #  okapi_login_token_response = requests.post(url_auth,
    #    data=request_token_payload, timeout=5)
    #  okapi_login_token_response.raise_for_status()
    #  okapi_login_token = okapi_login_token_response.json()

    self.okapi_login = \
    {
      "token": okapi_login_token,
      "url": url,
      "accessTime": str(time.time()),
      "header":
      {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "access_token": okapi_login_token["access_token"],
        "expires_in": str(okapi_login_token["expires_in"]),
        "token_type": okapi_login_token["token_type"],
        "scope": okapi_login_token["scope"]
      }
    }

    #except requests.exceptions.HTTPError as err:
    #  print(err)
    #  # we now that 403 is probably wrong password:
    #  if (okapi_login_token_response.status_code == 403):
    #    response = okapi_login_token_response.json()
    #    status["text"] = ("Got error from Auth0: " + response["error"] +
    #      ": " + response["error_description"])
    #    status["type"] = "FATAL"
    #    status["web_status"] = okapi_login_token_response.status_code
    #  else:
    #    status["text"] = "Got HTTPError when sending request. "
    #    status["type"] = "FATAL"
    #    status["web_status"] = okapi_login_token_response.status_code
    #  return status
    #except requests.exceptions.Timeout as err:
    #  print(err)
    #  status["text"] = "Got timeout when sending request. "
    #  status["type"] = "FATAL"
    #  status["web_status"] = 408  # correct timeout?
    #  return status
    #except requests.exceptions.RequestException as err:
    #  print(err)
    #  status["text"] = "Got unknown exception. "
    #  status["type"] = "FATAL"
    #  status["web_status"] = 520  # non-standard
    #  return status

    self.tdms = Tdms(self.okapi_login, self.url)

    return connector_response





class Tdms:


  def __init__(self, okapi_login, url):

    self.okapi_login = okapi_login
    self.url = url + "/tdms"


  def get(self):
    #print("jup")
    return Okapi.handle_rest_call(lambda: requests.get(self.url,
      headers = self.okapi_login["header"], timeout = 5))


  def add(self, file_path, file_format):

    with open(file_path, "r") as tdm_file:

      data = \
      {
        "tdm":
        {
          "type": "tdm." + file_format,
          "content": tdm_file.read()
        }
      }
      #print(data)
      return Okapi.handle_rest_call(lambda: requests.post(self.url, json = data,
        headers = self.okapi_login["header"], timeout = 5))




#!/usr/bin/python

import time
import json

import okapi

import sys

okapi = okapi.Okapi()
# later: space_monitor = space_monitor.SpaceMonitor()
response = okapi.init(YOUR_LOGIN_AS_STRING, YOUR_PASSWORD_AS_STRING)

print("Initialization result status: " + str(response["status"]))
if response["status"]["type"] == "FATAL":
  sys.exit(1)

response = okapi.tdms.get()

print("TDM-Get result status: " + str(response["status"]))
#if response["status"]["type"] == "FATAL":
#  sys.exit(1)
print("TDM-Get result data: " + str(response["actual_response"]))

response = okapi.tdms.add(TDM_KVN_TXT_FILE_PATH_AS_STRING, "txt")

print("TDM-Add result status: " + str(response["status"]))
#if response["status"]["type"] == "FATAL":
#  sys.exit(1)
print("TDM-Add result data: " + str(response["actual_response"]))

response = okapi.tdms.get()

print("TDM-Get result status: " + str(response["status"]))
#if response["status"]["type"] == "FATAL":
#  sys.exit(1)
print("TDM-Get result data: " + str(response["actual_response"]))





import requests
import json


def okapi_delete_object(okapi_login, object_to_delete, url_endpoint, max_retries=3):
    # okapi_add_object() Add (post) one object to the platform
    #
    #   Inputs
    #       okapi_login - Dict, containing at least URL, options and Token for
    #       okapi. Can be obtained using OkapiInit().
    #       object_to_delete - The object (eg. satellite) to be deleted (thus: the message).
    #       url_endpoint - the url, where-to send the request
    #
    #   Outputs
    #       response_json - dict containing the added object, if successful. Else empty
    #       error - Dict containing error information. Always check
    #               error['status'] If it is 'FATAL' something went very wrong,
    #               'WARNING's are less critical and 'NONE' or 'INFO' are no
    #               concern. error['message'] gives some explanation on the
    #               status, error['web_status'] gives the http response.

    # check the type that is requested

    # init
    response = dict()
    error = dict()
    error['message'] = 'NONE'
    error['status'] = 'NONE'
    error['web_status'] = 0

    if "satellite_id" in object_to_delete:
        url = okapi_login["url"] + url_endpoint + object_to_delete["satellite_id"]
    elif "conjunction_id" in object_to_delete:
        url = okapi_login["url"] + url_endpoint + object_to_delete["conjunction_id"]
    else:
        error['message'] = 'Object to delete appears incomplete. Missing id.'
        error['status'] = 'FATAL'
        error['web_status'] = 204
        return response, error

    retries = 1
    response_json = dict()
    while retries <= max_retries:
        try:
            response = requests.delete(url, data=json.dumps(object_to_delete),
                                       headers=okapi_login['header'], timeout=5)
            # raise for status
            response.raise_for_status()
            break

        except requests.exceptions.HTTPError as e:
            print("Exception: " + str(e))
            print("Response Body: {}".format(response.json()))

            # if we got a 500, we received an internal error.This we would like to
            # look at
            if response.status_code == 500:
                response_json = response.json()
                status = response_json['state_msg']
                error['message'] = status['text']
                error['status'] = status['type']
            else:
                error['message'] = 'Got HTTPError when sending request: ' + str(e)
                # DEBUG
                # print("HTTP Response " + str(response.json()))
                error['status'] = 'FATAL'
            error['web_status'] = response.status_code
            return response_json, error
        except requests.exceptions.Timeout as e:
            if retries == max_retries:
                error['message'] = 'Got timeout when sending request: ' + str(e)
                error['status'] = 'FATAL'
                error['web_status'] = 408
                return response_json, error
            retries += 1
            continue
        except requests.exceptions.RequestException as e:
            error['message'] = 'Got unknown exception (Wrong url?): ' + str(e)
            error['status'] = 'FATAL'
            error['web_status'] = 520  # non-standard
            return response_json, error

    # apparently, all when smoothly. Get the responses
    # DEBUG
    # print("HTTP Response " + str(response.json()))
    response_json = response.json()

    # fill error
    error['web_status'] = response.status_code

    # fill request -- depending on what has been called!
    # request['id'] = response_json['request_id']

    return response_json, error



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="okapi-python-connector",
    version="2021-02",
    author="Jonas Radtke",
    author_email="jonas@okapiorbits.space",
    description="Package to connect to OKAPI Api",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/OKAPIOrbits/OkapiPythonConnector",
    packages=setuptools.find_packages(),
    install_requires=[
        'requests'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
